# ============================================================
# DocumentInterface.py
# provides methods for document management to all apps.
# ------------------------------------------------------------
# Copyright (C) 2004-2005 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import QDateTime,QFile,QFileInfo,QMessageBox,QString,Qt,IO_ReadOnly,IO_WriteOnly
from qtsql import QSqlQuery,QSqlDatabase
from binascii import hexlify
from ProgressDialog import KProgressDialog


# ------------------------------------------------------------



def getFolderByRelation(aRelType, aRelID):
    quFolder = kumula.DB.doSelect("documents_folders", ("id",), "", {"reltype":aRelType,"relid":aRelID})
    found = 0
    while quFolder.next():
        myFolderID = quFolder.value(0).toInt()
        found = 1
    if (found > 0):
        return KFolder(myFolderID)
    else:
        return None


def createFolderRoot(aRelType, aRelID, aTitle=''):
    kumula.DB.doInsert("documents_folders", {"parentid":"0","title":aTitle,"reltype":aRelType,"relid":aRelID})
    myFolderID = kumula.DB.lastAutoID()
    return KFolder(myFolderID)


class KFolder(object):

    def __init__(self, anID):
        self.folderID = anID
        if (self.folderID > 0):
            quFolder = kumula.DB.doSelect("documents_folders", ("parentid","title","reltype","relid","link"), "", {"id":"%i" % self.folderID})
            while quFolder.next():
                self.parentID = quFolder.value(0).toInt()
                self.title    = quFolder.value(1).toString()
                self.relType  = quFolder.value(2).toString()
                self.relID    = quFolder.value(3).toString()
                self.link     = quFolder.value(4).toInt()
        else:
                self.parentID = 0
                self.title    = ""
                self.relType  = ""
                self.relID    = ""
                self.link     = 0



    def newDocument(self, aRelID, aTitle, aComment, aLogDate, someKeywords):
        myQuery = kumula.DB.doInsert("documents", {"folderid":"%i" % self.folderID, "relid":aRelID, "title":aTitle, "comment":aComment, "logdate":aLogDate.toString(Qt.ISODate), "keywords":someKeywords})
        myID = kumula.DB.lastAutoID()
        if (myID > 0):
            return KDocument(myID)
        else:
            return None



    def getChildren(self):
        if (self.link > 0):
            myChildren = KFolder(self.link).getChildren()
        else:
            myChildren = ()
            quFolders = kumula.DB.doSelect("documents_folders", ("id",), "title", {"parentid":"%i" % self.folderID})
            while quFolders.next():
                fldID = quFolders.value(0).toInt()
                myChildren = myChildren + (KFolder(fldID),)
        return myChildren


    def getDocuments(self, aRelID=None):
        myDocs = ()
        if (aRelID):
            quDocs = kumula.DB.doSelect("documents", ("id",), "title", {"folderid":"%i" % self.folderID, "relid":aRelID})
        else:
            quDocs = kumula.DB.doSelect("documents", ("id",), "title", {"folderid":"%i" % self.folderID})
        while quDocs.next():
            docID = quDocs.value(0).toInt()
            myDocs = myDocs + (KDocument(docID),)
        return myDocs


    def getDocumentCount(self, aRelID=None):
        if (aRelID):
            count = kumula.DB.doCount("documents", {"folderid":"%i" % self.folderID, "relid":aRelID})
        else:
            count = kumula.DB.doCount("documents", {"folderid":"%i" % self.folderID})
        return count


    def clearDocuments(self, aRelID=None):
        if (aRelID):
            kumula.DB.doDelete("documents", {"folderid":"%i" % self.folderID, "relid":aRelID})
        else:
            kumula.DB.doDelete("documents", {"folderid":"%i" % self.folderID})



# ------------------------------------------------------------



class KDocument(object):

    def __init__(self, anID):
        self.docID = anID
        if (self.docID > 0):
            quDoc = kumula.DB.doSelect("documents", ("folderid","relid","title","comment","logdate","keywords"), "", {"id":"%i" % self.docID})
            while quDoc.next():
                self.folderID = quDoc.value(0).toInt()
                self.relID    = quDoc.value(1).toString()
                self.title    = quDoc.value(2).toString()
                self.comment  = quDoc.value(3).toString()
                self.logDate  = quDoc.value(4).toDate()
                self.keywords = quDoc.value(5).toString()
        else:
                self.folderID = 0
                self.relID    = ""
                self.title    = ""
                self.comment  = ""
                self.logDate  = 0
                self.keywords = ""


    def newVersion(self, aComment):
        myQuery = kumula.DB.doInsert("documents_versions",
                   {"docid":"%i" % self.docID,
                    "time":QDateTime().currentDateTime().toString(Qt.ISODate),
                    "mimetype":"",
                    "extension":"",
                    "comment":aComment,
                    "user":kumula.UserName,
                    "filesize":0,
                    "checksum":"",
                    "compression":0})
        myID = kumula.DB.lastAutoID()
        if (myID > 0):
            return KDocVersion(myID)
        else:
            return None


    def importVersionFromFile(self, fromFile, aComment):
        myVersion = self.newVersion(aComment)
        if (myVersion):
            success = myVersion.loadFromFile(fromFile)
            if (success > 0):
                return myVersion
            else:
                myQuery = kumula.DB.doDelete("documents_versions", {"id":"%i" % myVersion.versionID})
                return None



    def getVersions(self):
        myVersions = ()
        quVersions = kumula.DB.doSelect("documents_versions", ("id",), "time", {"docid":"%i" % self.docID})
        while quVersions.next():
            versionID = quVersions.value(0).toInt()
            myVersions = myVersions + (KDocVersion(versionID),)
        return myVersions


    def getLatestVersion(self):
        lastVersion = 0
        quVersions = kumula.DB.doSelect("documents_versions", ("id",), "time", {"docid":"%i" % self.docID})
        while quVersions.next():
            lastVersion = quVersions.value(0).toInt()
        if (lastVersion > 0):
            return KDocVersion(lastVersion)
        else:
            return None


    def saveToFile(self, toFile):
        latestVersion = self.getLatestVersion()
        latestVersion.saveToFile(toFile)


    def moveToFolder(self, newFolderID):
        kumula.DB.doUpdate("documents", {"folderid":"%i" % newFolderID}, {"id":"%i" % self.docID}, 1)


    def delete(self):
        myVersions = self.getVersions()
        for version in myVersions:
            version.delete()
        kumula.DB.doDelete("documents", {"id":"%i" % self.docID})
        self = None


class KDocVersion(object):

    def __init__(self, anID):
        self.versionID = anID
        if (self.versionID > 0):
            quVersion = kumula.DB.doSelect("documents_versions", ("docid","time","mimetype","extension","comment","user","filesize","checksum","compression"), "", {"id":"%i" % self.versionID})
            while quVersion.next():
                self.docID       = quVersion.value(0).toInt()
                self.time        = quVersion.value(1).toDateTime()
                self.mimetype    = quVersion.value(2).toString()
                self.extension   = quVersion.value(3).toString()
                self.comment     = quVersion.value(4).toString()
                self.username    = quVersion.value(5).toString()
                self.filesize    = quVersion.value(6).toInt()
                self.checksum    = quVersion.value(7).toString()
                self.compression = quVersion.value(8).toInt()
        else:
            self.docID       = 0
            self.time        = 0
            self.mimetype    = ""
            self.extension   = ""
            self.comment     = ""
            self.username    = ""
            self.filesize    = 0
            self.checksum    = ""
            self.compression = 0


    def formatHexData(self, data):
        myHex = "0x" + hexlify(data)
        return myHex

    def loadFromFile(self, fromFile):
        blockSize = 65535

        myProgDlg = KProgressDialog(kumula.MainWin, "Importing file", fromFile)
        myProgDlg.show()

        myFile = QFile(fromFile)
        myFileInfo = QFileInfo(myFile)
        myProgDlg.setTarget(myFileInfo.size())

        if (myFile.open(IO_ReadOnly)):
            rank = 0
            data = myFile.readBlock(blockSize)
            while (data):
                myProgDlg.setPosition(rank * blockSize)
                myStatement = QString("INSERT INTO documents_binaries (versionid,rank,data) VALUES ('")
                myStatement.append("%i" % self.versionID)
                myStatement.append("','")
                myStatement.append("%i" % rank)
                myStatement.append("', ")
                # convert data for better database compatibility
                myStatement.append(self.formatHexData(data))
                myStatement.append(" );")
                quInsert = kumula.DB.execStatement(myStatement)
                rank = rank + 1
                data = myFile.readBlock(blockSize)
            myFile.close()
            myProgDlg.hide()
            # SUCCESS!
            return 1
        else:
            myProgDlg.hide()
            QMessageBox.warning(kumula.MainWin, "Can't open file", "The file couldn't be opened for reading!")
            # FAILED!
            return 0


    def saveToFile(self, toFile):
        myProgDlg = KProgressDialog(kumula.MainWin, "Exporting file", toFile)
        myProgDlg.show()
        myFile = QFile(toFile)
        Pieces = kumula.DB.doCount("documents_binaries", {"versionid":"%i" % self.versionID})
        myProgDlg.setTarget(Pieces)

        piece = 0
        if (myFile.open(IO_WriteOnly)):
            quData = kumula.DB.doSelect("documents_binaries", ("data",), "rank", {"versionid":"%i" % self.versionID})
            while quData.next():
                myFile.writeBlock(quData.value(0).toByteArray())
                piece = piece + 1
                myProgDlg.setPosition(piece)
            myFile.close()

            myProgDlg.hide()
            myMessage = QString("The file was successfully exported to\n")
            myMessage.append(toFile)
            QMessageBox.information(kumula.MainWin, "Info", myMessage)
        else:
            myProgDlg.hide()
            QMessageBox.warning(kumula.MainWin, "Can't open file", "The file couldn't be opened for writing!")



    def delete(self):
        kumula.DB.doDelete("documents_binaries", {"versionid":"%i" % self.versionID})
        kumula.DB.doDelete("documents_versions", {"id":"%i" % self.versionID})
        self = None


# ------------------------------------------------------------



class KDocumentInterface(object):

    def __init__(self, aDB):
        self.DB = aDB

    def newFolder(self, title, parentID, clientOnly=0):
        pass

    def moveFolder(self, id, newParentID):
        pass

    def renameFolder(self, id, newTitle):
        pass

    def deleteFolder(self, id):
        pass

    def getFolders(self, parentID=0):
        myTuple = ()
        quFolders = self.DB.doSelect("documents_folders", ("id","title","clientid"), "title", {"parentid":parentID})
        while quFolders.next():
            fldID     = quFolders.value(0).toString()
            fldTitle  = quFolders.value(1).toString()
            fldClient = quFolders.value(2).toString()
            displayIT = 0
            # Only use this folder, if...
            if (fldClient == ""):
                # ...it is a GLOBAL folder, or...
                displayIT = 1
            else:
                if (fldClient == kumula.ClientID):
                    # ...if it's a CLIENT folder of the selected client
                    displayIT = 1
            if (displayIT == 1):
                if (fldClient == ""):
                    myFolder = (fldID, fldTitle, "g")
                else:
                    myFolder = (fldID, fldTitle, "c")
                myTuple = myTuple + (myFolder,)



    def doQueryAllDocuments(self, values):
        quDocs = self.DB.doSelect("documents", values, "",
                     {"clientid":kumula.ClientID})
        return quDocs


    def getDocumentsOfFolder(self, folderID, values):
        quDocs = self.DB.doSelect("documents", values, "",
                     {"clientid":kumula.ClientID, "folderid":folderID})
        myDocs = ()
        while quDocs.next():
            myItem = ()
            for i in range(len(values)):
                myValue = quDocs.value(i).toString()
                myItem = myItem + (myValue,)
            myDocs = myDocs + (myItem,)
        return myDocs


    def getAllDocuments(self, values):
        myValues = ("folderid",) + values
        quDocs = self.DB.doSelect("documents", myValues, "",
                      {"clientid":kumula.ClientID})
        myDocs = ()
        while quDocs.next():
            myItem = ()
            myFolderID = quDocs.value(0).toInt()
            if (myFolderID >= 0):
                # negative folderIDs have special meaning,
                # e.g. -1 (inbox) or -9 (trashbin)
                for i in range(1, len(myValues)):
                    myValue = quDocs.value(i).toString()
                    myItem = myItem + (myValue,)
                myDocs = myDocs + (myItem,)
        return myDocs


    def getDocumentsOfInbox(self, values):
        return self.getDocumentsOfFolder("-1", values)


    def getDocumentsOfTrashbin(self, values):
        return self.getDocumentsOfFolder("-9", values)


    def getDocCount(self, folderID):
        pass


    def emptyTrashbin(self):
        self.DB.doDelete("documents", {"clientid":kumula.ClientID, "folderid":"-9"})


    def moveDocumentToFolder(self, docID, folderID):
        self.DB.doUpdate("documents", {"folderid":folderID}, {"id":docID}, 1)

    def moveDocumentToInbox(self, docID):
        self.moveDocumentToFolder(docID, "-1")

    def moveDocumentToTrash(self, docID):
        self.moveDocumentToFolder(docID, "-9")


    def formatHexData(self, data):
        myHex = "0x" + hexlify(data)
        return myHex



    def importFile(self, fromFile, vID):
        blockSize = 65535

        myProgDlg = KProgressDialog(kumula.MainWin, "Importing file", fromFile)
        myProgDlg.show()

        myFile = QFile(fromFile)
        myFileInfo = QFileInfo(myFile)
        myProgDlg.setTarget(myFileInfo.size())

        if (myFile.open(IO_ReadOnly)):
            rank = 0
            data = myFile.readBlock(blockSize)
            while (data):
                myProgDlg.setPosition(rank * blockSize)
                myStatement = QString("INSERT INTO documents_binaries (versionid,rank,data) VALUES ('")
                myStatement.append(vID)
                myStatement.append("','")
                myStatement.append("%i" % rank)
                myStatement.append("', ")
                # convert data for better database compatibility
                myStatement.append(self.formatHexData(data))
                myStatement.append(" );")
                quInsert = self.DB.execStatement(myStatement)
                rank = rank + 1
                data = myFile.readBlock(blockSize)
            myFile.close()
            myProgDlg.hide()
            return 1
        else:
            myProgDlg.hide()
            QMessageBox.warning(kumula.MainWin, "Can't open file", "The file couldn't be opened for reading!")
            return 0


